/**
*	Author:	Elizabeth Labelle
*	ID:			17417
*	Project:	Boids Simulation
*	Date:		December 2010
*
*	Description:	The AttackBoid class defines behavior for predator boids. This script should be
*					attached to the AttackBoidPrefab and AltAttackBoid prefabs (difference between the
*					prefabs is color).
*
*					Predator boids have two states: Hungry and Full. When a predator boid is hungry, it will search
*					for the nearest normal boid and follow it until the predator boid is close enough to eat it. Eating
*					a normal boid removes it from the world. The predator boid then becomes full.
*
*					When a predator boid is full, it will wander around the world (staying within world boundaries) and
*					ignore other boids. After a period of time, the boid will become hungry again.
*/

using UnityEngine;
using System.Collections;

public class AttackBoid : MonoBehaviour {

	/** How fast the boid can move. Exposed to the Unity editor so it can be modified during runtime. **/
	public int speed = 8;
	
	/** The position of the AttackBoid's target (either a random position or a boid) in world coordinates **/
	internal Vector3 targetPos;
	/** The Boid the AttackBoid is currently following (null if not following) **/
	internal GameObject target;
	/** True if AttackBoid is Hungry and should follow nearest Boid, False if Full **/
	internal bool isFull = false;
	
	/**
	*	Called when the AttackBoid object is first instantiated by the Unity Engine.
	*	Searches the world to find the nearest Boid and sets it as the target. The AttackBoid will begin following it.
	*	Also makes the AttackBoid hungry.
	*/
	void Start () 
	{
		if(tag == "" || tag == null)
			tag = "AttackBoid";
		isFull = false;
		generateTarget();
	}
	
	/**
	*	Called once per frame by the Unity Engine.
	*	Moves the AttackBoid closer to its target position. When the AttackBoid is close to its position, the
	*	AttackBoid eats the Boid at that position (if currently following one), then generates a new target position.
	*/
	void Update () 
	{
		//find the closest boid (if one exists) and move to it
		//otherwise move to a random location in the scene
		if(!isFull)
		{
			generateTarget();
		}
		transform.position += transform.TransformDirection(Vector3.forward) * speed * Time.deltaTime;
		
		//eat the boid if close enough
		if((transform.position - targetPos).magnitude < 5)
		{
			//look at the target so the AttackBoid can move forward towards it
			transform.LookAt(Vector3.forward);
			if(target != null)
			{
				//if following a Boid, the AttackBoid is close enough to eat it, do so
				StartCoroutine(eatBoid());
			}
		}
	}
	
	/**
	*	Finds the closest Boid in the scene. The closest Boid is then set as the target. The targetPos is set to the
	*	position of the new target. If no Boids exist in the scene then a random location is instead chosen with
	*	findRandomLocation()
	*/
	void generateTarget()
	{
		//get all Boids in the scene
		GameObject[] targets = GameObject.FindGameObjectsWithTag("Boid");
		
		//if there are no targets left, pick a new targetPosition
		if(targets.Length <= 0)
		{
			//only pick a new target if we have reached the old one
			if((transform.position - targetPos).magnitude < 2)
			{
				targetPos = findRandomLocation();
			}
			target = null;
			transform.LookAt(targetPos);
			return;
		}
		else
		{
			Vector3 closestTarget = Vector3.zero, currTarget;
			float closestDistSqr = Mathf.Infinity, currDistSqr;
			GameObject closestObject = null;
			
			//Iterate through all boids in the scene to find the closest one
			foreach(GameObject t in targets)
			{
				currTarget = t.transform.position;
				//calculate the distance between the AttackBoid and the current Boid
				currDistSqr = (currTarget - transform.position).sqrMagnitude;
				
				//if the current boid is closest, save its information
				if(currDistSqr < closestDistSqr)
				{
					closestObject = t;
					closestTarget = t.transform.position;
					closestDistSqr = currDistSqr;
				}
			}
			
			target = closestObject;
			targetPos = closestTarget;
			//turn to face the target so the AttackBoid can move forward to follow it
			transform.LookAt(targetPos);
			return;
		}
	}
	
	/**
	*	Returns the position of a random location in the environment.
	*	Does not consider Boids in the scene when generating.
	*/
	Vector3 findRandomLocation()
	{
		//get the WorldData from the scene
		GameObject worldObject = GameObject.FindWithTag("WorldData");
		
		if(worldObject == null)
		{
			return Vector3.zero;
		}
		else
		{
			World worldData = worldObject.GetComponent<World>();
			
			return new Vector3(Random.Range(worldData.xMin, worldData.xMax), 
										 Random.Range(worldData.yMin, worldData.yMax), 
										 Random.Range(worldData.zMin, worldData.zMax));
		}	
	}

	/**
	*	eatBoid() is a Unity coroutine.
	*	Method is called when an AttackBoid gets close enough to the target Boid to eat it.
	*	If the AttackBoid has a valid Boid target, its beEaten() method is called, which deletes
	*	the Boid object. The Boid GameObject is then destroyed, the AttackBoid made full, and a new
	*	target position is chosen.
	*	The coroutine handles the timer event causing the AttackBoid to become hungry again after time has passed.
	*	A check is then made to see if the eaten boid was the last one in the scene, if yes the program will
	*	restart.
	*/
	IEnumerator eatBoid()
	{
		Boid bTarget = target.GetComponent<Boid>();
		
		if(bTarget != null)
		{			
			//call the Boids beEaten() method to handle removing the object
			bTarget.beEaten();
			//delete the GameObject
			Destroy(target);
			target = null;
			
			isFull = true;
			
			//AttackBoid is full, wander to a random location instead of attacking other Boids
			target = null;
			targetPos = findRandomLocation();
			transform.LookAt(targetPos);
			
			//wait for 10 seconds
			yield return new WaitForSeconds(10.0f);
			
			isFull = false;
			
			GameObject obj = GameObject.FindWithTag("WorldData");
			World world = obj.GetComponent<World>();
			world.checkRestart();
		}
	}
}
