/**
*	Author:	Elizabeth Labelle
*	ID:			17417
*	Project:	Boids Simulation
*	Date:		December 2010
*
*	Description:	The BoidManager spawns and keeps track of a number of Boids in the scene.
*					It also keeps track of variables such as min/max velocity, flock center, and average
*					velocity which each Boid references when calculating movement vectors.
*
*/
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class BoidManager : MonoBehaviour 
{
	//Editable values to change boid flock behavior
	/** The Boid GameObject containing the geometry, materials, and scripts for a Boid. 
	*	Must be set in the Editor.
	**/ 
	public Boid prefab;
	/** The number of Boids spawned by the BoidManager at startup. **/
	public int numBoids = 30;
	/** The maximum velocity any Boid can move at. Exposed to the Unity editor 
	*	so it can be modified during runtime.
	**/
	public int maxVelocity = 20;
	/** The minimum velocity any Boid can move at. Exposed to the Unity editor 
	*	so it can be modified during runtime.
	**/
	public int minVelocity = 12;
	/** The minimum distance each Boid must keep away from other Boids.
	*	 Exposed to the Unity editor so it can be modified during runtime. 
	**/
	public int minDist = 2;
	/** The minimum distance each Boid must keep away from the ground.
	*	 Exposed to the Unity editor so it can be modified during runtime. 
	**/
	public int minDistGround = 15;
	/**
	*	Radius of the sphere that Boids are spawned inside during startup.
	*	 Exposed to the Unity editor so it can be modified during runtime. 
	*/
	public int spawnRadius = 15;
	
	/** A list of all boids managed by this BoidManager */
	internal List<Boid> boidList = new List<Boid>();
	/** Average position of all Boids in the flock. */
	internal Vector3 boidCenter;
	/** Average velocity of all Boids in the flock. */
	internal Vector3 boidVelocity;
	
	/**
	*	Start method is called on initialization.
	*	Creates boids in the scene inside the BoidManager's collider.
	*	Uses the numBoids variable to determine how many boids to create.
	*/
	void Start () 
	{
		for(int i = 0; i < numBoids; i++) 
		{
			Boid boid = Instantiate(prefab, transform.position, transform.rotation) as Boid;
			boid.transform.parent = transform;
			//generate random values for theta and phi
			float theta = 2 * Mathf.PI * Random.value;
			float phi = Mathf.Acos(2 * Random.value - 1);
			//calculate a point on the surface of the sphere. Then multiply by a value 0-1 to get a point inside the sphere (not just surface).
			//x = r * sin(theta) * cos(phi)
			//y = r * sin(theta) * sin(phi)
			//z = r * cos(theta)
			boid.transform.position += new Vector3(	Random.value * spawnRadius * Mathf.Sin(theta) * Mathf.Cos(phi),
																	Random.value * spawnRadius * Mathf.Sin(theta) * Mathf.Sin(phi),
																	Random.value *spawnRadius * Mathf.Cos(theta));
			boid.controller = this;
			boidList.Add(boid);
		}
	}
	
	/**
	*	Called every frame by the Unity Engine. 
	*	Recalculates the center and average velocity of the flock.
	*/
	void Update()
	{
		boidCenter = Vector3.zero;
		boidVelocity = Vector3.zero;
		
		foreach(Boid b in boidList)
		{
			boidCenter += b.rigidbody.position;
			boidVelocity += b.rigidbody.velocity;
		}
		
		boidCenter /= boidList.Count;
		boidVelocity /= boidList.Count;
	}
	
	/**
	*	Removes a Boid from the boidList.
	*/
	public void removeBoid(Boid b)
	{
		boidList.Remove(b);
	}
}
